//====================================
//    Fish Feeder by Ruben Sommer
//    
//    last updated at:
//    18/05/2020
//
// Feel free to use, modify or share this code
// Use at own risk! I am in no way responsible for 
// any problems, damages, overly fed fish, dead fish or anything else
// by using this code. If you like this project, please like the instructable as well!
//
// Visit the instructable at: <insert instructable link>
//====================================

#include <Servo.h>

// Set the servo pin
int servoPin = 9;

//TUNE FISH FEEDER
//Start with setting the servo position above the food dropping hole
int servoAngle_dropOff = 87;
//Set the servo position which is below the food container
int servoAngle_FoodContainer = 141; 
//Set an idle servo position in between these two, to let both holes be closed off when no food is needed
//Change it into the food container position if you don't want to use it.
int servoAngle_idle = 115;

//FOOD SERVINGS
//First set the number of portions per feeding time
int nrFoodPortions = 2;

//Set the current time in 24h format (This will be seen as the starting point when the arduino is turned on!)
//As no clock module is used, time is tracked by setting an initial time
//To be as accurate as possible, connect the arduino usb power 2 seconds before this initial time
int initialTime[3] = {15, 00, 00};

//Set your feeding times here
//Simply uses an array of integers which represent the time
//Set the times according to:
//
//                       h   m    s
//int feeding_times[] = {13, 00, 00}
//Which is equivalent to a single feeding time at 13.00 (1 p.m.)
//
//Note:
//Do not set feeding times too close ( < 10sec apart)
//Change the number of food portions instead
//Besides the fact that I would not know why you would want to do this,
//setting the times this close can cause a feeding time to be skipped as it takes some time
//to feed a portion.
//                     h   m    s
int feeding_times[] = {15, 00, 00,
                       15, 05, 00,
                       15, 10, 00,
                       15, 15, 00,
                       15, 20, 00,
                       15, 25, 00,
                       15, 30, 00,
                       15, 35, 00,
                       15, 40, 00,
                       15, 45, 00,
                       15, 50, 00,
                       15, 55, 00,
                       16, 00, 00}; 
//=====================================================================
//=====================================================================
//Initializing variables, DO NOT CHANGE anything below this!
//(unless you know what you are doing :) )
Servo servo;  
double t0 = millis();
double previousTime = t0;
double currentTime = t0;

int hours, minutes, seconds;


bool getFood = false;
bool dropFood = false;
bool idle;
int portionCounter = 0;
int sec = 1000;
 
void setup()
{
  Serial.begin(9600);  
  servo.attach(servoPin);
  idle = true;
  servo.write(servoAngle_idle);

  hours = initialTime[0];
  minutes = initialTime[1];
  seconds = initialTime[2];
}
 
 
void loop()
{ 

  
  //Update current time
  updateTime();

  //Check if it is feeding time
  checkFeedTime();

  //Drop food if needed
  DropFood();
  

};

//End of main loop
//===========================================
void updateTime(){
  currentTime = millis();
  if(currentTime - previousTime >= 1*sec){
    seconds++;
    previousTime = currentTime;

    //Check to update minutes
    if(seconds == 60){
      minutes++;
      seconds = 0;

      //Check to update hours
      if(minutes == 60){
        hours++;
        minutes = 0;

        if(hours == 24){
          hours = 0;
        };//End hours check
      }; //End of min check
    }; //End of sec check

    //Print time
    Serial.print("Current Time: ");
    Serial.print(hours);
    Serial.print(":");
    Serial.print(minutes);
    Serial.print(":");
    Serial.println(seconds);
  };

  //End of time update
};

void checkFeedTime(){
  int nrFeedingTimes = sizeof(feeding_times)/sizeof(int);

  for(int i = 0; i < nrFeedingTimes; i = i + 3){
      if(hours == feeding_times[i]){
        if(minutes == feeding_times[i+1]){
          if(seconds == feeding_times[i+2]){
            //Time to feed!
            getFood = true;
          };
        };
      };
  };
};

void DropFood(){
  double t;
 //Checks current state and provides food if needed
  if(getFood == true){
    t = millis();
    Serial.println("State: getFood");
    servo.write(servoAngle_FoodContainer);

    getFood = false;
    dropFood = true;

    //Delay 1 sec using millis()
    while(millis() - t < 1*sec){
      //Do nothing for one second
    };
  };
  
  if(dropFood == true){
    t = millis();
    
    Serial.print("State: dropFood");
    servo.write(servoAngle_dropOff);
    dropFood = false;
    portionCounter++;

    //Delay 1 sec using millis()
    while(millis() - t < 2*sec){
      //Do nothing for one second
    };
    
    if(portionCounter < nrFoodPortions){
      Serial.println("  Switch to getFood");
      getFood = true;
    };
    if(portionCounter >= nrFoodPortions){
      Serial.println("  Switch to idle");
      idle = true;
      servo.write(servoAngle_idle);
      portionCounter=0;
    };

  };
}
